"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InvalidTypeError = exports.UnexpectedArgumentAfterVariadicArgumentError = exports.UnexpectedRequiredArgumentError = exports.MissingRequiredOptionError = exports.DependingOptionError = exports.ConflictingOptionError = exports.OptionNotCombinableError = exports.UnexpectedOptionValueError = exports.InvalidOptionValueError = exports.MissingOptionValueError = exports.UnknownOptionError = exports.InvalidOptionError = exports.DuplicateOptionError = exports.ValidationError = exports.UnknownTypeError = exports.UnknownConflictingOptionError = exports.UnknownRequiredOptionError = exports.FlagsError = void 0;
const _utils_js_1 = require("./_utils.js");
class FlagsError extends Error {
    constructor(message) {
        super(message);
        Object.setPrototypeOf(this, FlagsError.prototype);
    }
}
exports.FlagsError = FlagsError;
class UnknownRequiredOptionError extends FlagsError {
    constructor(option, options) {
        super(`Unknown required option "${(0, _utils_js_1.getFlag)(option)}".${(0, _utils_js_1.didYouMeanOption)(option, options)}`);
        Object.setPrototypeOf(this, UnknownRequiredOptionError.prototype);
    }
}
exports.UnknownRequiredOptionError = UnknownRequiredOptionError;
class UnknownConflictingOptionError extends FlagsError {
    constructor(option, options) {
        super(`Unknown conflicting option "${(0, _utils_js_1.getFlag)(option)}".${(0, _utils_js_1.didYouMeanOption)(option, options)}`);
        Object.setPrototypeOf(this, UnknownConflictingOptionError.prototype);
    }
}
exports.UnknownConflictingOptionError = UnknownConflictingOptionError;
class UnknownTypeError extends FlagsError {
    constructor(type, types) {
        super(`Unknown type "${type}".${(0, _utils_js_1.didYouMeanType)(type, types)}`);
        Object.setPrototypeOf(this, UnknownTypeError.prototype);
    }
}
exports.UnknownTypeError = UnknownTypeError;
/* Validation errors. */
/**
 * A validation error is thrown when the command is wrongly used by the user.
 * For example: If the user passes some invalid options or arguments to the
 * command.
 */
class ValidationError extends FlagsError {
    constructor(message) {
        super(message);
        Object.setPrototypeOf(this, ValidationError.prototype);
    }
}
exports.ValidationError = ValidationError;
class DuplicateOptionError extends ValidationError {
    constructor(name) {
        super(`Option "${(0, _utils_js_1.getFlag)(name).replace(/^--no-/, "--")}" can only occur once, but was found several times.`);
        Object.setPrototypeOf(this, DuplicateOptionError.prototype);
    }
}
exports.DuplicateOptionError = DuplicateOptionError;
class InvalidOptionError extends ValidationError {
    constructor(option, options) {
        super(`Invalid option "${(0, _utils_js_1.getFlag)(option)}".${(0, _utils_js_1.didYouMeanOption)(option, options)}`);
        Object.setPrototypeOf(this, InvalidOptionError.prototype);
    }
}
exports.InvalidOptionError = InvalidOptionError;
class UnknownOptionError extends ValidationError {
    constructor(option, options) {
        super(`Unknown option "${(0, _utils_js_1.getFlag)(option)}".${(0, _utils_js_1.didYouMeanOption)(option, options)}`);
        Object.setPrototypeOf(this, UnknownOptionError.prototype);
    }
}
exports.UnknownOptionError = UnknownOptionError;
class MissingOptionValueError extends ValidationError {
    constructor(option) {
        super(`Missing value for option "${(0, _utils_js_1.getFlag)(option)}".`);
        Object.setPrototypeOf(this, MissingOptionValueError.prototype);
    }
}
exports.MissingOptionValueError = MissingOptionValueError;
class InvalidOptionValueError extends ValidationError {
    constructor(option, expected, value) {
        super(`Option "${(0, _utils_js_1.getFlag)(option)}" must be of type "${expected}", but got "${value}".`);
        Object.setPrototypeOf(this, InvalidOptionValueError.prototype);
    }
}
exports.InvalidOptionValueError = InvalidOptionValueError;
class UnexpectedOptionValueError extends ValidationError {
    constructor(option, value) {
        super(`Option "${(0, _utils_js_1.getFlag)(option)}" doesn't take a value, but got "${value}".`);
        Object.setPrototypeOf(this, InvalidOptionValueError.prototype);
    }
}
exports.UnexpectedOptionValueError = UnexpectedOptionValueError;
class OptionNotCombinableError extends ValidationError {
    constructor(option) {
        super(`Option "${(0, _utils_js_1.getFlag)(option)}" cannot be combined with other options.`);
        Object.setPrototypeOf(this, OptionNotCombinableError.prototype);
    }
}
exports.OptionNotCombinableError = OptionNotCombinableError;
class ConflictingOptionError extends ValidationError {
    constructor(option, conflictingOption) {
        super(`Option "${(0, _utils_js_1.getFlag)(option)}" conflicts with option "${(0, _utils_js_1.getFlag)(conflictingOption)}".`);
        Object.setPrototypeOf(this, ConflictingOptionError.prototype);
    }
}
exports.ConflictingOptionError = ConflictingOptionError;
class DependingOptionError extends ValidationError {
    constructor(option, dependingOption) {
        super(`Option "${(0, _utils_js_1.getFlag)(option)}" depends on option "${(0, _utils_js_1.getFlag)(dependingOption)}".`);
        Object.setPrototypeOf(this, DependingOptionError.prototype);
    }
}
exports.DependingOptionError = DependingOptionError;
class MissingRequiredOptionError extends ValidationError {
    constructor(option) {
        super(`Missing required option "${(0, _utils_js_1.getFlag)(option)}".`);
        Object.setPrototypeOf(this, MissingRequiredOptionError.prototype);
    }
}
exports.MissingRequiredOptionError = MissingRequiredOptionError;
class UnexpectedRequiredArgumentError extends ValidationError {
    constructor(arg) {
        super(`An required argument cannot follow an optional argument, but "${arg}"  is defined as required.`);
        Object.setPrototypeOf(this, UnexpectedRequiredArgumentError.prototype);
    }
}
exports.UnexpectedRequiredArgumentError = UnexpectedRequiredArgumentError;
class UnexpectedArgumentAfterVariadicArgumentError extends ValidationError {
    constructor(arg) {
        super(`An argument cannot follow an variadic argument, but got "${arg}".`);
        Object.setPrototypeOf(this, UnexpectedArgumentAfterVariadicArgumentError.prototype);
    }
}
exports.UnexpectedArgumentAfterVariadicArgumentError = UnexpectedArgumentAfterVariadicArgumentError;
class InvalidTypeError extends ValidationError {
    constructor({ label, name, value, type }, expected) {
        super(`${label} "${name}" must be of type "${type}", but got "${value}".` + (expected
            ? ` Expected values: ${expected.map((value) => `"${value}"`).join(", ")}`
            : ""));
        Object.setPrototypeOf(this, MissingOptionValueError.prototype);
    }
}
exports.InvalidTypeError = InvalidTypeError;
