"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseFlags = void 0;
const _utils_js_1 = require("./_utils.js");
const _errors_js_1 = require("./_errors.js");
const deprecated_js_1 = require("./deprecated.js");
const boolean_js_1 = require("./types/boolean.js");
const number_js_1 = require("./types/number.js");
const string_js_1 = require("./types/string.js");
const _validate_flags_js_1 = require("./_validate_flags.js");
const integer_js_1 = require("./types/integer.js");
const DefaultTypes = {
    string: string_js_1.string,
    number: number_js_1.number,
    integer: integer_js_1.integer,
    boolean: boolean_js_1.boolean,
};
/**
 * Parse command line arguments.
 * @param argsOrCtx Command line arguments e.g: `Deno.args` or parse context.
 * @param opts      Parse options.
 *
 * ```
 * // examples/flags/flags.ts -x 3 -y.z -n5 -abc --beep=boop foo bar baz --deno.land --deno.com -- --cliffy
 * parseFlags(Deno.args);
 * ```
 *
 * Output:
 *
 * ```
 * {
 *   flags: {
 *     x: "3",
 *     y: { z: true },
 *     n: "5",
 *     a: true,
 *     b: true,
 *     c: true,
 *     beep: "boop",
 *     deno: { land: true, com: true }
 *   },
 *   literal: [ "--cliffy" ],
 *   unknown: [ "foo", "bar", "baz" ],
 *   stopEarly: false,
 *   stopOnUnknown: false
 * }
 * ```
 */
function parseFlags(argsOrCtx, opts = {}) {
    let args;
    let ctx;
    if (Array.isArray(argsOrCtx)) {
        ctx = {};
        args = argsOrCtx;
    }
    else {
        ctx = argsOrCtx;
        args = argsOrCtx.unknown;
        argsOrCtx.unknown = [];
    }
    args = args.slice();
    ctx.flags ??= {};
    ctx.literal ??= [];
    ctx.unknown ??= [];
    ctx.stopEarly = false;
    ctx.stopOnUnknown = false;
    opts.dotted ??= true;
    validateOptions(opts);
    const options = parseArgs(ctx, args, opts);
    (0, _validate_flags_js_1.validateFlags)(ctx, opts, options);
    if (opts.dotted) {
        parseDottedOptions(ctx);
    }
    return ctx;
}
exports.parseFlags = parseFlags;
function validateOptions(opts) {
    opts.flags?.forEach((opt) => {
        opt.depends?.forEach((flag) => {
            if (!opts.flags || !(0, _utils_js_1.getOption)(opts.flags, flag)) {
                throw new _errors_js_1.UnknownRequiredOptionError(flag, opts.flags ?? []);
            }
        });
        opt.conflicts?.forEach((flag) => {
            if (!opts.flags || !(0, _utils_js_1.getOption)(opts.flags, flag)) {
                throw new _errors_js_1.UnknownConflictingOptionError(flag, opts.flags ?? []);
            }
        });
    });
}
function parseArgs(ctx, args, opts) {
    /** Option name mapping: propertyName -> option.name */
    const optionsMap = new Map();
    let inLiteral = false;
    for (let argsIndex = 0; argsIndex < args.length; argsIndex++) {
        let option;
        let current = args[argsIndex];
        let currentValue;
        let negate = false;
        // literal args after --
        if (inLiteral) {
            ctx.literal.push(current);
            continue;
        }
        else if (current === "--") {
            inLiteral = true;
            continue;
        }
        else if (ctx.stopEarly || ctx.stopOnUnknown) {
            ctx.unknown.push(current);
            continue;
        }
        const isFlag = current.length > 1 && current[0] === "-";
        if (!isFlag) {
            if (opts.stopEarly) {
                ctx.stopEarly = true;
            }
            ctx.unknown.push(current);
            continue;
        }
        const isShort = current[1] !== "-";
        const isLong = isShort ? false : current.length > 3 && current[2] !== "-";
        if (!isShort && !isLong) {
            throw new _errors_js_1.InvalidOptionError(current, opts.flags ?? []);
        }
        // normalize short flags: -abc => -a -b -c
        if (isShort && current.length > 2 && current[2] !== ".") {
            args.splice(argsIndex, 1, ...splitFlags(current));
            current = args[argsIndex];
        }
        else if (isLong && current.startsWith("--no-")) {
            negate = true;
        }
        // split value: --foo="bar=baz" => --foo bar=baz
        const equalSignIndex = current.indexOf("=");
        if (equalSignIndex !== -1) {
            currentValue = current.slice(equalSignIndex + 1) || undefined;
            current = current.slice(0, equalSignIndex);
        }
        if (opts.flags) {
            option = (0, _utils_js_1.getOption)(opts.flags, current);
            if (!option) {
                const name = current.replace(/^-+/, "");
                option = (0, _utils_js_1.matchWildCardOptions)(name, opts.flags);
                if (!option) {
                    if (opts.stopOnUnknown) {
                        ctx.stopOnUnknown = true;
                        ctx.unknown.push(args[argsIndex]);
                        continue;
                    }
                    throw new _errors_js_1.UnknownOptionError(current, opts.flags);
                }
            }
        }
        else {
            option = {
                name: current.replace(/^-+/, ""),
                optionalValue: true,
                type: deprecated_js_1.OptionType.STRING,
            };
        }
        if (option.standalone) {
            ctx.standalone = option;
        }
        const positiveName = negate
            ? option.name.replace(/^no-?/, "")
            : option.name;
        const propName = (0, _utils_js_1.paramCaseToCamelCase)(positiveName);
        if (typeof ctx.flags[propName] !== "undefined") {
            if (!opts.flags?.length) {
                option.collect = true;
            }
            else if (!option.collect) {
                throw new _errors_js_1.DuplicateOptionError(current);
            }
        }
        if (option.type && !option.args?.length) {
            option.args = [{
                    type: option.type,
                    requiredValue: option.requiredValue,
                    optionalValue: option.optionalValue,
                    variadic: option.variadic,
                    list: option.list,
                    separator: option.separator,
                }];
        }
        if (opts.flags?.length && !option.args?.length &&
            typeof currentValue !== "undefined") {
            throw new _errors_js_1.UnexpectedOptionValueError(option.name, currentValue);
        }
        let optionArgsIndex = 0;
        let inOptionalArg = false;
        const next = () => currentValue ?? args[argsIndex + 1];
        const previous = ctx.flags[propName];
        parseNext(option);
        if (typeof ctx.flags[propName] === "undefined") {
            if (option.args?.[optionArgsIndex]?.requiredValue) {
                throw new _errors_js_1.MissingOptionValueError(option.name);
            }
            else if (typeof option.default !== "undefined") {
                ctx.flags[propName] = (0, _utils_js_1.getDefaultValue)(option);
            }
            else {
                ctx.flags[propName] = true;
            }
        }
        if (option.value) {
            ctx.flags[propName] = option.value(ctx.flags[propName], previous);
        }
        else if (option.collect) {
            const value = typeof previous !== "undefined"
                ? (Array.isArray(previous) ? previous : [previous])
                : [];
            value.push(ctx.flags[propName]);
            ctx.flags[propName] = value;
        }
        optionsMap.set(propName, option);
        opts.option?.(option, ctx.flags[propName]);
        /** Parse next argument for current option. */
        // deno-lint-ignore no-inner-declarations
        function parseNext(option) {
            if (negate) {
                ctx.flags[propName] = false;
                return;
            }
            else if (!option.args?.length) {
                ctx.flags[propName] = undefined;
                return;
            }
            const arg = option.args[optionArgsIndex];
            if (!arg) {
                const flag = next();
                throw new _errors_js_1.UnknownOptionError(flag, opts.flags ?? []);
            }
            if (!arg.type) {
                arg.type = deprecated_js_1.OptionType.BOOLEAN;
            }
            if (option.args?.length && !option.type) {
                // make all values required by default
                if ((typeof arg.optionalValue === "undefined" ||
                    arg.optionalValue === false) &&
                    typeof arg.requiredValue === "undefined") {
                    arg.requiredValue = true;
                }
            }
            else {
                // make non boolean value required by default
                if (arg.type !== deprecated_js_1.OptionType.BOOLEAN &&
                    (typeof arg.optionalValue === "undefined" ||
                        arg.optionalValue === false) &&
                    typeof arg.requiredValue === "undefined") {
                    arg.requiredValue = true;
                }
            }
            if (!arg.requiredValue) {
                inOptionalArg = true;
            }
            else if (inOptionalArg) {
                throw new _errors_js_1.UnexpectedRequiredArgumentError(option.name);
            }
            let result;
            let increase = false;
            if (arg.list && hasNext(arg)) {
                const parsed = next()
                    .split(arg.separator || ",")
                    .map((nextValue) => {
                    const value = parseValue(option, arg, nextValue);
                    if (typeof value === "undefined") {
                        throw new _errors_js_1.InvalidOptionValueError(option.name, arg.type ?? "?", nextValue);
                    }
                    return value;
                });
                if (parsed?.length) {
                    result = parsed;
                }
            }
            else {
                if (hasNext(arg)) {
                    result = parseValue(option, arg, next());
                }
                else if (arg.optionalValue && arg.type === deprecated_js_1.OptionType.BOOLEAN) {
                    result = true;
                }
            }
            if (increase && typeof currentValue === "undefined") {
                argsIndex++;
                if (!arg.variadic) {
                    optionArgsIndex++;
                }
                else if (option.args[optionArgsIndex + 1]) {
                    throw new _errors_js_1.UnexpectedArgumentAfterVariadicArgumentError(next());
                }
            }
            if (typeof result !== "undefined" &&
                (option.args.length > 1 || arg.variadic)) {
                if (!ctx.flags[propName]) {
                    ctx.flags[propName] = [];
                }
                ctx.flags[propName].push(result);
                if (hasNext(arg)) {
                    parseNext(option);
                }
            }
            else {
                ctx.flags[propName] = result;
            }
            /** Check if current option should have an argument. */
            function hasNext(arg) {
                if (!option.args?.length) {
                    return false;
                }
                const nextValue = currentValue ?? args[argsIndex + 1];
                if (!nextValue) {
                    return false;
                }
                if (option.args.length > 1 && optionArgsIndex >= option.args.length) {
                    return false;
                }
                if (arg.requiredValue) {
                    return true;
                }
                // require optional values to be called with an equal sign: foo=bar
                if (option.equalsSign && arg.optionalValue && !arg.variadic &&
                    typeof currentValue === "undefined") {
                    return false;
                }
                if (arg.optionalValue || arg.variadic) {
                    return nextValue[0] !== "-" ||
                        typeof currentValue !== "undefined" ||
                        (arg.type === deprecated_js_1.OptionType.NUMBER && !isNaN(Number(nextValue)));
                }
                return false;
            }
            /** Parse argument value.  */
            function parseValue(option, arg, value) {
                const result = opts.parse
                    ? opts.parse({
                        label: "Option",
                        type: arg.type || deprecated_js_1.OptionType.STRING,
                        name: `--${option.name}`,
                        value,
                    })
                    : parseDefaultType(option, arg, value);
                if (typeof result !== "undefined") {
                    increase = true;
                }
                return result;
            }
        }
    }
    return optionsMap;
}
function parseDottedOptions(ctx) {
    // convert dotted option keys into nested objects
    ctx.flags = Object.keys(ctx.flags).reduce((result, key) => {
        if (~key.indexOf(".")) {
            key.split(".").reduce((
            // deno-lint-ignore no-explicit-any
            result, subKey, index, parts) => {
                if (index === parts.length - 1) {
                    result[subKey] = ctx.flags[key];
                }
                else {
                    result[subKey] = result[subKey] ?? {};
                }
                return result[subKey];
            }, result);
        }
        else {
            result[key] = ctx.flags[key];
        }
        return result;
    }, {});
}
function splitFlags(flag) {
    flag = flag.slice(1);
    const normalized = [];
    const index = flag.indexOf("=");
    const flags = (index !== -1 ? flag.slice(0, index) : flag).split("");
    if (isNaN(Number(flag[flag.length - 1]))) {
        flags.forEach((val) => normalized.push(`-${val}`));
    }
    else {
        normalized.push(`-${flags.shift()}`);
        if (flags.length) {
            normalized.push(flags.join(""));
        }
    }
    if (index !== -1) {
        normalized[normalized.length - 1] += flag.slice(index);
    }
    return normalized;
}
function parseDefaultType(option, arg, value) {
    const type = arg.type || deprecated_js_1.OptionType.STRING;
    const parseType = DefaultTypes[type];
    if (!parseType) {
        throw new _errors_js_1.UnknownTypeError(type, Object.keys(DefaultTypes));
    }
    return parseType({
        label: "Option",
        type,
        name: `--${option.name}`,
        value,
    });
}
